/*
 * PWM_Luefter_Auto.c
 *
 * Created: 02.02.2018 11:52:04
 *  Author: peter
 */ 
#include <stdlib.h>

#include <avr/io.h>
#include <avr/interrupt.h>

#define DIR_CCW			0x10
#define DIR_CW			0x20
#define DIR_NONE		0x00
#define MAX_SPEED		250
#define LOW_SPEED		0
#define CUT_OFF_SPEED	40
#define START_SPEED		60

void wait_ms(uint8_t);
void pwm_init(void);
void init_timer( void );
uint8_t get_encoder(void);
void eeprom_schreiben(uint8_t wert);
uint8_t eeprom_lesen(void);
void ramp(void);

volatile uint8_t event=0;					// Encoder events
volatile uint8_t pwm_value;
extern uint8_t key_press;
volatile uint8_t npwm_value;
volatile uint8_t power_state;				// Start with power off state

int main(void)
{
	
	
	
	
	uint8_t enc_input=0;			// Encoder input
	uint8_t nval=0;
	
	pwm_value = 0;					// Initial state is off
	npwm_value = 0;					// none set	
	power_state = 0;				// Off
	

	
	DDRD &= ~((1 << PD4) | ( 1 << PD5) | ( 1 << PD1));		// Inputs		PD4 and PD for encoder input, PD1 for power key
	PORTD |= (1 << PD4) | ( 1 << PD5);		// Pull up on
		
	
	
	init_timer();		// ticks


	sei();				// enable interrupts
	
	
	wait_ms(200);
	
	pwm_init();
				
	key_press = 0;
			
		while(1)
		{

				if ( power_state == 1 && (key_press & ( 1 << PD1)) ){		// power off
					
					eeprom_schreiben(pwm_value);			// save last speed in eeprom
					power_state = 0;
					pwm_value=0;
					key_press = 0;
					continue;
				}
				if ( power_state == 0 && (key_press & ( 1 << PD1)) ){		// set power on
					
					npwm_value = eeprom_lesen();					// read last speed from eeprom
						if ( npwm_value == 0 || npwm_value == 255)
								npwm_value = START_SPEED;			// if not set yet set START_SPEED
					power_state = 1;
					key_press = 0;
					ramp();
					continue;
				}
				
				enc_input =  get_encoder();
				
				if ( enc_input & DIR_CW){
					if ( npwm_value != MAX_SPEED){
							if ( npwm_value < MAX_SPEED && npwm_value >= CUT_OFF_SPEED){
								
								npwm_value += 10;
								nval=1;
								} else {
											npwm_value = CUT_OFF_SPEED;
											nval=1;
							}
					}
				}
				if ( enc_input & DIR_CCW){
					if ( npwm_value > CUT_OFF_SPEED){
						
						npwm_value -= 10;
						nval=1;
					} else {
							npwm_value = 0;
							nval = 1;
					}
						
				}
				if ( nval ){
					
					ramp();
					
					nval=0;
					
				}
		}
    
}
void ramp(void)
{
			
			if ( npwm_value < pwm_value) { // Ramp down
				
								
				while (  npwm_value < pwm_value){
					pwm_value--;
					
					wait_ms(20);
					if ( pwm_value < CUT_OFF_SPEED){
						
							pwm_value = 0;
							return;
					}
				}
			} else {
		
								
				while( pwm_value < npwm_value){ // Ramp up
					pwm_value++;
					wait_ms(20);			// wait for stabilization of current
				}
		
				
			}
		
}
void eeprom_schreiben(uint8_t wert){
	
	cli();
	
	while(EECR & (1<<EEPE));		/* Wait for completion of previous write */
	
	EEAR = 0;		// Adresse 0
	EEDR = wert;	// was
	EECR &= ~((1 << EEPM1)| ( 1 << EEPM0)); // Set Mode Erase, than write the byte
	EECR |= (1<<EEMPE);
	
								/* Start eeprom write by setting EEPE */
	EECR |= (1<<EEPE);	
	sei();
}
uint8_t eeprom_lesen(void){
	
	cli();
	
	/* Wait for completion of previous write */
	while(EECR & (1<<EEPE))
	;
	/* Set up address register */
	EEAR = 0;
	/* Start eeprom read by writing EERE */
	EECR |= (1<<EERE);
	/* Return data from data register */
	sei();
	return EEDR;
}