// rotary.h 
#include <avr/io.h>
#include <avr/interrupt.h>

// ENCODER
#define PHASE_B			(PIND & (1<<PD4))	// 
#define PHASE_A			(PIND & (1<<PD5))	// 

#define DIR_CCW			0x10
#define DIR_CW			0x20
#define DIR_NONE		0x00

#define R_START			0x00

#define HALF_STEP		

#ifdef HALF_STEP
// Use the half-step state table (emits a code at 00 and 11)
#define R_CCW_BEGIN		0x1
#define R_CW_BEGIN		0x2
#define R_START_M		0x3
#define R_CW_BEGIN_M	0x4
#define R_CCW_BEGIN_M	0x5

const uint8_t ttable[6][4] = {
	// R_START (00)
	{R_START_M,            R_CW_BEGIN,     R_CCW_BEGIN,  R_START},
	// R_CCW_BEGIN
	{R_START_M | DIR_CCW, R_START,        R_CCW_BEGIN,  R_START},
	// R_CW_BEGIN
	{R_START_M | DIR_CW,  R_CW_BEGIN,     R_START,      R_START},
	// R_START_M (11)
	{R_START_M,            R_CCW_BEGIN_M,  R_CW_BEGIN_M, R_START},
	// R_CW_BEGIN_M
	{R_START_M,            R_START_M,      R_CW_BEGIN_M, R_START | DIR_CW},
	// R_CCW_BEGIN_M
	{R_START_M,            R_CCW_BEGIN_M,  R_START_M,    R_START | DIR_CCW},
};
#else
// Use the full-step state table (emits a code at 00 only)
#define R_CW_FINAL		0x1
#define R_CW_BEGIN		0x2
#define R_CW_NEXT		0x3
#define R_CCW_BEGIN		0x4
#define R_CCW_FINAL		0x5
#define R_CCW_NEXT		0x6

const uint8_t ttable[7][4] = {
	// R_START
	{R_START,    R_CW_BEGIN,  R_CCW_BEGIN, R_START},
	// R_CW_FINAL
	{R_CW_NEXT,  R_START,     R_CW_FINAL,  R_START | DIR_CW},
	// R_CW_BEGIN
	{R_CW_NEXT,  R_CW_BEGIN,  R_START,     R_START},
	// R_CW_NEXT
	{R_CW_NEXT,  R_CW_BEGIN,  R_CW_FINAL,  R_START},
	// R_CCW_BEGIN
	{R_CCW_NEXT, R_START,     R_CCW_BEGIN, R_START},
	// R_CCW_FINAL
	{R_CCW_NEXT, R_CCW_FINAL, R_START,     R_START | DIR_CCW},
	// R_CCW_NEXT
	{R_CCW_NEXT, R_CCW_FINAL, R_CCW_BEGIN, R_START},
};
#endif
// rotary.h END

//extern volatile uint8_t	event;				// encoder events


/* 2 ms */
//ISR(TIMER1_COMPA_vect)
uint8_t get_encoder(void)
{
//...
			static uint8_t rot_state = 0;	// encoder	
			uint8_t pinstate=0;				// Encoder
			uint8_t event=0;
//.....


		/*** encoder ***************************************/
		if (PHASE_A )
			pinstate |= 0x01;
		if (PHASE_B)
			pinstate |= 0x02;
		rot_state = ttable[rot_state & 0x0F][pinstate];
		
		if ((rot_state & 0x30) == DIR_CW)
		  event |= DIR_CW;

		if ((rot_state & 0x30) == DIR_CCW)
		  event |= DIR_CCW;	
		  
		  return(event);
		  
} // END ISR

// Anwendungsseite

// Lesen des events vom ENCODER
/*
uint8_t encoder_read( void )
{
	uint8_t r;
	
	asm("cli");
	r = event;
	event &= ~event;		// clear only that bit
	asm("sei");
	return r;
}
*/